@@ -12,6 +12,7 @@
 #include <string.h>
 
 #include "xb-silo-private.h"
+#include "xb-string-private.h"
 #include "xb-builder.h"
 #include "xb-builder-fixup-private.h"
 #include "xb-builder-source-private.h"
@@ -149,28 +150,35 @@ xb_builder_compile_text_cb (GMarkupParseContext *context,
 {
 	XbBuilderCompileHelper *helper = (XbBuilderCompileHelper *) user_data;
 	XbBuilderNode *bn = helper->current;
-	guint i;
-
-	/* no data */
-	if (text_len == 0)
-		return;
+	XbBuilderNode *bc = xb_builder_node_get_last_child (bn);
 
 	/* unimportant */
 	if (xb_builder_node_has_flag (bn, XB_BUILDER_NODE_FLAG_IGNORE))
 		return;
 
-	/* all whitespace? */
-	for (i = 0; i < text_len; i++) {
-		if (!g_ascii_isspace (text[i]))
-			break;
-	}
-	if (i >= text_len)
-		return;
-
 	/* repair text unless we know it's valid */
 	if (helper->source_flags & XB_BUILDER_SOURCE_FLAG_LITERAL_TEXT)
 		xb_builder_node_add_flag (bn, XB_BUILDER_NODE_FLAG_LITERAL_TEXT);
-	xb_builder_node_set_text (bn, text, text_len);
+
+	/* text or tail */
+	if (!xb_builder_node_has_flag (bn, XB_BUILDER_NODE_FLAG_HAS_TEXT)) {
+		xb_builder_node_set_text (bn, text, text_len);
+		return;
+	}
+
+	/* does this node have a child */
+	if (bc != NULL) {
+		xb_builder_node_set_tail (bc, text, text_len);
+		return;
+	}
+	if (!xb_builder_node_has_flag (bn, XB_BUILDER_NODE_FLAG_HAS_TAIL)) {
+		xb_builder_node_set_tail (bn, text, text_len);
+		return;
+	}
+	g_set_error (error,
+		     G_IO_ERROR,
+		     G_IO_ERROR_INVALID_DATA,
+		     "Mismatched XML; cannot store %s", text);
 }
 
 /**
@@ -350,10 +358,14 @@ xb_builder_strtab_text_cb (XbBuilderNode *bn, gpointer user_data)
 		return FALSE;
 	if (xb_builder_node_has_flag (bn, XB_BUILDER_NODE_FLAG_IGNORE))
 		return FALSE;
-	if (xb_builder_node_get_text (bn) == NULL)
-		return FALSE;
-	tmp = xb_builder_node_get_text (bn);
-	xb_builder_node_set_text_idx (bn, xb_builder_compile_add_to_strtab (helper, tmp));
+	if (xb_builder_node_get_text (bn) != NULL) {
+		tmp = xb_builder_node_get_text (bn);
+		xb_builder_node_set_text_idx (bn, xb_builder_compile_add_to_strtab (helper, tmp));
+	}
+	if (xb_builder_node_get_tail (bn) != NULL) {
+		tmp = xb_builder_node_get_tail (bn);
+		xb_builder_node_set_tail_idx (bn, xb_builder_compile_add_to_strtab (helper, tmp));
+	}
 	return FALSE;
 }
 
@@ -447,8 +459,18 @@ xb_builder_nodetab_write_node (XbBuilderNodetabHelper *helper, XbBuilderNode *bn
 		.next		= 0x0,
 		.parent		= 0x0,
 		.text		= xb_builder_node_get_text_idx (bn),
+		.tail		= xb_builder_node_get_tail_idx (bn),
 	};
 
+	/* if the node had no children and the text is just whitespace then
+	 * remove it even in literal mode */
+	if (xb_builder_node_has_flag (bn, XB_BUILDER_NODE_FLAG_LITERAL_TEXT)) {
+		if (xb_string_isspace (xb_builder_node_get_text (bn), -1))
+			sn.text = XB_SILO_UNSET;
+		if (xb_string_isspace (xb_builder_node_get_tail (bn), -1))
+			sn.tail = XB_SILO_UNSET;
+	}
+
 	/* save this so we can set up the ->next pointers correctly */
 	xb_builder_node_set_offset (bn, helper->buf->len);
 
@@ -554,37 +576,17 @@ xb_builder_compile_helper_free (XbBuilderCompileHelper *helper)
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(XbBuilderCompileHelper, xb_builder_compile_helper_free)
 
-static void
-_uuid_generate_sha1 (uuid_t out, const uuid_t ns, const char *name, size_t len)
-{
-#ifdef HAVE_UUID_GENERATE_SHA1
-	uuid_generate_sha1 (out, ns, name, len);
-#else
-	guint8 buf[20];
-	gsize bufsz = sizeof(buf);
-	g_autoptr(GChecksum) csum = g_checksum_new (G_CHECKSUM_SHA1);
-	g_checksum_update (csum, (const guchar *) name, (gssize) len);
-	g_checksum_get_digest (csum, buf, &bufsz);
-	memcpy (out, buf, sizeof(uuid_t));
-#endif
-}
-
 static gchar *
 xb_builder_generate_guid (XbBuilder *self)
 {
 	XbBuilderPrivate *priv = GET_PRIVATE (self);
-	uuid_t guid;
-	gchar guid_tmp[UUID_STR_LEN] = { '\0' };
-
-	if (priv->guid->len == 0) {
-		uuid_clear (guid);
-	} else {
-		uuid_t ns;
-		uuid_clear (ns);
-		_uuid_generate_sha1 (guid, ns, priv->guid->str, priv->guid->len);
+	XbGuid guid = { 0x0 };
+	if (priv->guid->len > 0) {
+		xb_guid_compute_for_data (&guid,
+					  (const guint8 *) priv->guid->str,
+					  priv->guid->len);
 	}
-	uuid_unparse (guid, guid_tmp);
-	return g_strdup (guid_tmp);
+	return xb_guid_to_string (&guid);
 }
 
 /**
@@ -815,9 +817,11 @@ xb_builder_compile (XbBuilder *self, XbBuilderCompileFlags flags, GCancellable *
 	/* add the initial header */
 	hdr.strtab = nodetabsz;
 	if (priv->guid->len > 0) {
-		uuid_t ns;
-		uuid_clear (ns);
-		_uuid_generate_sha1 (hdr.guid, ns, priv->guid->str, priv->guid->len);
+		XbGuid guid_tmp;
+		xb_guid_compute_for_data (&guid_tmp,
+					  (const guint8 *) priv->guid->str,
+					  priv->guid->len);
+		memcpy (&hdr.guid, &guid_tmp, sizeof(guid_tmp));
 	}
 	XB_SILO_APPENDBUF (buf, &hdr, sizeof(XbSiloHeader));
 
