@@ -6,31 +6,26 @@
 
 #pragma once
 
-#include <uuid.h>
-
 #include "xb-machine.h"
 #include "xb-node.h"
 #include "xb-silo.h"
 
-G_BEGIN_DECLS
+#include "xb-string-private.h"
 
-/* for old versions of libuuid */
-#ifndef UUID_STR_LEN
-#define UUID_STR_LEN	37
-#endif
+G_BEGIN_DECLS
 
 /* 32 bytes, native byte order */
 typedef struct __attribute__ ((packed)) {
 	guint32		magic;
 	guint32		version;
-	uuid_t		guid;
+	XbGuid		guid;
 	guint16		strtab_ntags;
 	guint8		padding[2];
 	guint32		strtab;
 } XbSiloHeader;
 
 #define XB_SILO_MAGIC_BYTES		0x624c4d58
-#define XB_SILO_VERSION			0x00000006
+#define XB_SILO_VERSION			0x00000007
 #define XB_SILO_UNSET			0xffffffff
 
 typedef struct __attribute__ ((packed)) {
@@ -40,6 +35,7 @@ typedef struct __attribute__ ((packed)) {
 	guint32		parent;		/* ONLY when is_node: from 0 */
 	guint32		next;		/* ONLY when is_node: from 0 */
 	guint32		text;		/* ONLY when is_node: from strtab */
+	guint32		tail;		/* ONLY when is_node: from strtab */
 } XbSiloNode;
 
 typedef struct __attribute__ ((packed)) {
@@ -82,6 +78,8 @@ const gchar	*xb_silo_node_get_element	(XbSilo		*self,
 						 XbSiloNode	*n);
 const gchar	*xb_silo_node_get_text		(XbSilo		*self,
 						 XbSiloNode	*n);
+const gchar	*xb_silo_node_get_tail		(XbSilo		*self,
+						 XbSiloNode	*n);
 XbSiloAttr	*xb_silo_node_get_attr_by_str	(XbSilo		*self,
 						 XbSiloNode	*n,
 						 const gchar	*name);
