@@ -331,6 +331,10 @@ xb_silo_to_string (XbSilo *self, GError **error)
 				g_string_append_printf (str, "text:         %s\n",
 							xb_silo_from_strtab (self, n->text));
 			}
+			if (n->tail != XB_SILO_UNSET) {
+				g_string_append_printf (str, "tail:         %s\n",
+							xb_silo_from_strtab (self, n->tail));
+			}
 			for (guint8 i = 0; i < n->nr_attrs; i++) {
 				XbSiloAttr *a = xb_silo_get_attr (self, off, i);
 				g_string_append_printf (str, "attr_name:    %s [%03u]\n",
@@ -369,6 +373,15 @@ xb_silo_node_get_text (XbSilo *self, XbSiloNode *n)
 	return xb_silo_from_strtab (self, n->text);
 }
 
+/* private */
+const gchar *
+xb_silo_node_get_tail (XbSilo *self, XbSiloNode *n)
+{
+	if (n->tail == XB_SILO_UNSET)
+		return NULL;
+	return xb_silo_from_strtab (self, n->tail);
+}
+
 /* private */
 const gchar *
 xb_silo_node_get_element (XbSilo *self, XbSiloNode *n)
@@ -576,10 +589,10 @@ xb_silo_get_machine (XbSilo *self)
 gboolean
 xb_silo_load_from_bytes (XbSilo *self, GBytes *blob, XbSiloLoadFlags flags, GError **error)
 {
+	XbGuid guid_tmp;
 	XbSiloHeader *hdr;
 	XbSiloPrivate *priv = GET_PRIVATE (self);
 	gsize sz = 0;
-	gchar guid[UUID_STR_LEN] = { '\0' };
 	guint32 off = 0;
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->nodes_mutex);
 	g_autoptr(GTimer) timer = g_timer_new ();
@@ -632,8 +645,8 @@ xb_silo_load_from_bytes (XbSilo *self, GBytes *blob, XbSiloLoadFlags flags, GErr
 	}
 
 	/* get GUID */
-	uuid_unparse (hdr->guid, guid);
-	priv->guid = g_strdup (guid);
+	memcpy (&guid_tmp, &hdr->guid, sizeof(guid_tmp));
+	priv->guid = xb_guid_to_string (&guid_tmp);
 
 	/* check strtab */
 	priv->strtab = hdr->strtab;
@@ -722,6 +735,9 @@ xb_silo_watch_file_cb (GFileMonitor *monitor,
 {
 	XbSilo *silo = XB_SILO (user_data);
 	g_autofree gchar *fn = g_file_get_path (file);
+	g_autofree gchar *basename = g_file_get_basename (file);
+	if (g_str_has_prefix (basename, ".goutputstream"))
+		return;
 	g_debug ("%s changed, invalidating", fn);
 	xb_silo_invalidate (silo);
 }
@@ -1055,6 +1071,31 @@ xb_silo_machine_func_text_cb (XbMachine *self,
 	return TRUE;
 }
 
+static gboolean
+xb_silo_machine_func_tail_cb (XbMachine *self,
+			      XbStack *stack,
+			      gboolean *result,
+			      gpointer user_data,
+			      gpointer exec_data,
+			      GError **error)
+{
+	XbSilo *silo = XB_SILO (user_data);
+	XbSiloQueryData *query_data = (XbSiloQueryData *) exec_data;
+
+	/* optimize pass */
+	if (query_data == NULL) {
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED,
+				     "cannot optimize: no silo to query");
+		return FALSE;
+	}
+	xb_machine_stack_push_steal (self, stack,
+				     xb_opcode_new (XB_OPCODE_KIND_INDEXED_TEXT,
+						    xb_silo_node_get_tail (silo, query_data->sn),
+						    query_data->sn->tail,
+						    NULL));
+	return TRUE;
+}
+
 static gboolean
 xb_silo_machine_func_first_cb (XbMachine *self,
 			       XbStack *stack,
@@ -1239,6 +1280,8 @@ xb_silo_init (XbSilo *self)
 			       xb_silo_machine_func_stem_cb, self, NULL);
 	xb_machine_add_method (priv->machine, "text", 0,
 			       xb_silo_machine_func_text_cb, self, NULL);
+	xb_machine_add_method (priv->machine, "tail", 0,
+			       xb_silo_machine_func_tail_cb, self, NULL);
 	xb_machine_add_method (priv->machine, "first", 0,
 			       xb_silo_machine_func_first_cb, self, NULL);
 	xb_machine_add_method (priv->machine, "last", 0,
