@@ -4,15 +4,14 @@
  * SPDX-License-Identifier: LGPL-2.1+
  */
 
-#ifndef __XB_STACK_H
-#define __XB_STACK_H
-
-G_BEGIN_DECLS
+#pragma once
 
 #include <glib-object.h>
 
 #include "xb-opcode.h"
 
+G_BEGIN_DECLS
+
 typedef struct _XbStack XbStack;
 
 GType		 xb_stack_get_type		(void);
@@ -24,5 +23,3 @@ gboolean	 xb_stack_push_steal		(XbStack	*self,
 						 XbOpcode	*opcode);
 
 G_END_DECLS
-
-#endif /* __XB_STACK_H */
